package com.fyber.mediation;

import android.app.Activity;
import com.fyber.mediation.adcolony.AdColonyMediationAdapter;
import com.fyber.mediation.applovin.AppLovinMediationAdapter;
import com.fyber.mediation.hyprmx.HyprMXMediationAdapter;
import com.fyber.mediation.vungle.VungleMediationAdapter;
import com.fyber.utils.FyberLogger;
import java.lang.InterruptedException;
import java.lang.Object;
import java.lang.String;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public final class MediationAdapterStarter {
  private static final String TAG = "MediationAdapterStarter";

  public static AdaptersListener adaptersListener;

  private static void startAppLovin(final Activity activity, final Map<String, Object> configs, final Map<String, MediationAdapter> map) {
    try {
      MediationAdapter adapter = new AppLovinMediationAdapter();
      FyberLogger.d(TAG, "Starting adapter AppLovin with version 6.3.0-r1");
      if (adapter.startAdapter(activity, configs)) {
        FyberLogger.d(TAG, "Adapter AppLovin with version 6.3.0-r1 was started successfully");
        map.put("applovin", adapter);
      } else {
        FyberLogger.d(TAG, "Adapter AppLovin with version 6.3.0-r1 was not started successfully");
      }
    } catch (Throwable throwable) {
      FyberLogger.e(TAG, "Exception occurred while loading adapter AppLovin with version 6.3.0-r1 - " + throwable.getCause());
    }
  }

  private static void startHyprMX(final Activity activity, final Map<String, Object> configs, final Map<String, MediationAdapter> map) {
    try {
      MediationAdapter adapter = new HyprMXMediationAdapter();
      FyberLogger.d(TAG, "Starting adapter HyprMX with version 4.0.1-r3");
      if (adapter.startAdapter(activity, configs)) {
        FyberLogger.d(TAG, "Adapter HyprMX with version 4.0.1-r3 was started successfully");
        map.put("hyprmx", adapter);
      } else {
        FyberLogger.d(TAG, "Adapter HyprMX with version 4.0.1-r3 was not started successfully");
      }
    } catch (Throwable throwable) {
      FyberLogger.e(TAG, "Exception occurred while loading adapter HyprMX with version 4.0.1-r3 - " + throwable.getCause());
    }
  }

  private static void startVungle(final Activity activity, final Map<String, Object> configs, final Map<String, MediationAdapter> map) {
    try {
      MediationAdapter adapter = new VungleMediationAdapter();
      FyberLogger.d(TAG, "Starting adapter Vungle with version 4.0.2-r1");
      if (adapter.startAdapter(activity, configs)) {
        FyberLogger.d(TAG, "Adapter Vungle with version 4.0.2-r1 was started successfully");
        map.put("vungle", adapter);
      } else {
        FyberLogger.d(TAG, "Adapter Vungle with version 4.0.2-r1 was not started successfully");
      }
    } catch (Throwable throwable) {
      FyberLogger.e(TAG, "Exception occurred while loading adapter Vungle with version 4.0.2-r1 - " + throwable.getCause());
    }
  }

  private static void startAdColony(final Activity activity, final Map<String, Object> configs, final Map<String, MediationAdapter> map) {
    try {
      MediationAdapter adapter = new AdColonyMediationAdapter();
      FyberLogger.d(TAG, "Starting adapter AdColony with version 2.3.6-r2");
      if (adapter.startAdapter(activity, configs)) {
        FyberLogger.d(TAG, "Adapter AdColony with version 2.3.6-r2 was started successfully");
        map.put("adcolony", adapter);
      } else {
        FyberLogger.d(TAG, "Adapter AdColony with version 2.3.6-r2 was not started successfully");
      }
    } catch (Throwable throwable) {
      FyberLogger.e(TAG, "Exception occurred while loading adapter AdColony with version 2.3.6-r2 - " + throwable.getCause());
    }
  }

  public static Map<String, MediationAdapter> startAdapters(final Activity activity, final Map<String, Map<String, Object>> configs) {
    Map<String, MediationAdapter> map = new HashMap<>();
    startAppLovin(activity, getConfigsForAdapter(configs, "AppLovin"), map);
    startHyprMX(activity, getConfigsForAdapter(configs, "HyprMX"), map);
    startVungle(activity, getConfigsForAdapter(configs, "Vungle"), map);
    startAdColony(activity, getConfigsForAdapter(configs, "AdColony"), map);
    return map;
  }

  public static int getAdaptersCount() {
    return 4;
  }

  private static Map<String, Object> getConfigsForAdapter(Map<String, Map<String, Object>> configs, String adapter) {
    Map<String, Object> config = configs.get(adapter.toLowerCase());
    if (config == null) {
      config = Collections.emptyMap();
    }
    return config;
  }

  private static Map<String, Map<String, Object>> getConfigs(final Future<Map<String, Map<String, Object>>> futureConfig) {
    Map<String, Map<String, Object>> configs = MediationConfigProvider.getConfigs();
    Map<String, Map<String, Object>> runtimeConfigs = MediationConfigProvider.getRuntimeConfigs();
    configs = mergeConfigs(configs, runtimeConfigs);
    try {
      if (futureConfig != null) {
        Map<String, Map<String, Object>> serverConfigs = futureConfig.get();
        configs = mergeConfigs(configs, serverConfigs);
      }
    } catch (InterruptedException | ExecutionException e) {
      FyberLogger.e(TAG, "Exception occurred", e);
    }
    return configs;
  }

  private static Map<String, Map<String, Object>> mergeConfigs(final Map<String, Map<String, Object>> intoConfigs, final Map<String, Map<String, Object>> fromConfigs) {
    if (fromConfigs != null && !fromConfigs.isEmpty()) {
      for (Map.Entry<String, Map<String, Object>> entry: fromConfigs.entrySet()) {
        String network = entry.getKey();
        Map<String, Object> adapterIntoConfigs = entry.getValue();
        Map<String, Object> adapterFromConfigs = intoConfigs.get(network);
        if (adapterFromConfigs != null) {
          adapterIntoConfigs.putAll(adapterFromConfigs);
        }
        intoConfigs.put(network, adapterIntoConfigs);
      }
    } else {
      FyberLogger.d(TAG, "There were no configurations to override");
    }
    return intoConfigs;
  }

  public static Map<String, MediationAdapter> startAdapters(final Activity activity, final Future<Map<String, Map<String, Object>>> future) {
    Map<String, Map<String, Object>> configs = getConfigs(future);
    Map<String, MediationAdapter> adapters = startAdapters(activity, configs);
    if (adaptersListener != null) {
      adaptersListener.startedAdapters(adapters.keySet(), configs);
    }
    return adapters;
  }
}
